package lsedit;

// Layout method using Coffman-Graham Layer Assignment algorithm and the
// Sugiyama algorithm.

// John S. Y. Lee January 2000

import java.util.Enumeration;
import java.util.Vector;

public class SugiyamaLayout extends LandscapeLayouter  implements ToolBarEventHandler {

  public SugiyamaLayout(LandscapeEditorCore ls) 
  {
	super(ls);
  }

/****************************************************************************/

  public String getMenuLabel() 
  {
	return "Layout using Coffman-Graham-Sugiyama algorithm";
  } // getMenuLabel

/***************************************************************************/

  // The doLayout method executes the Coffman-Graham Layer Assignment
  // algorithm and the Sugiyama algorithm on the boxes selected.
  // Assumption: All boxes selected are in the same container.

  // This is called directly to layout incoming TA

	public void doLayout1(Vector selectedBoxes, EntityInstance container, boolean update) 
	{
		int				width, height;
		Enumeration		en, edge;
		EntityInstance	e, child;
		int				vertexID = 0;
		int				groupSize = selectedBoxes.size();
		RelationInstance relation;
		RelationClass	 rc;
		int				index;

		width  = container.getWidth();
		height = container.getHeight();
		
		// create graph to store info on selected boxes and their relationships
		Graph graph = new Graph(groupSize);

		/* Graph is an array of vertex's */

		for (en = selectedBoxes.elements(); en.hasMoreElements(); ) {
			e = (EntityInstance) en.nextElement();
			graph.addVertex(e.widthRelLocal());
			e.orMark(EntityInstance.IN_GRAPH_MARK);
		}

		// get (directed) relationships between the boxes

		for (en = selectedBoxes.elements(); en.hasMoreElements(); ++vertexID) {
			e = (EntityInstance) en.nextElement();

			// Need only look at src since will be src for one or other node in diagram

			for (edge = e.srcRelationElements(); edge.hasMoreElements(); ) {
				relation = (RelationInstance) edge.nextElement();
				rc       = (RelationClass)    relation.getParentClass();
				if (rc == null || rc.isClassVisible()) {
					child = relation.getDst();
					if (child != e && child.isMarked(EntityInstance.IN_GRAPH_MARK)) {
						// store relationship only if children is in the same container
						for (index = 0; index < selectedBoxes.size(); index++) {
							if (child == selectedBoxes.elementAt(index)) {
								break;
							}
						} // for
						graph.addEdge(vertexID, index);
					}
				}	
			} // for
		} // for

		for (en = selectedBoxes.elements(); en.hasMoreElements(); ) {
			e = (EntityInstance) en.nextElement();
			e.nandMark(EntityInstance.IN_GRAPH_MARK);
		}

/*
		int index = 0;
	  
		System.out.println("Graph read:");
		for (en = selectedBoxes.elements(); en.hasMoreElements(); index++) {
			System.out.println(index + " " + en.nextElement());
		}
		graph.print();
*/

		Vector layers = graph.doCoffmanGrahamSugiyama();

/*
		System.out.println("RESULTS");
		int level = 0;
		for (en = layers.elements(); en.hasMoreElements(); ) {
			System.out.println("level " + level);
			for (Enumeration f = ((Vector)en.nextElement()).elements(); f.hasMoreElements(); ) {
				System.out.print(f.nextElement() + " ");
			}
			System.out.println();
		}
 */

		// Assign coordinates to each box

		int		numLayers   = layers.size();
		double	layerHeight = (double) 1.0 / (double)numLayers;
		double	xrel, yrel, widthrel, heightrel, boxWidth;
		Vector	row;

		double curX = 0.01;
		double curY = 1.0;

		for (en = layers.elements(); en.hasMoreElements(); ) {

			row       = (Vector) en.nextElement();
			boxWidth  = (double) 1.0 / (double)(row.size());

			for (Enumeration f = row.elements(); f.hasMoreElements(); ) {
				int curVertex = ((Integer)f.nextElement()).intValue();

				// Process non-dummy boxes only
				if (curVertex < groupSize) {
					e         = (EntityInstance) selectedBoxes.elementAt(curVertex);
					widthrel  = e.widthRelLocal();
					heightrel = e.heightRelLocal();
					xrel = curX + (boxWidth    - widthrel) / 2;
					yrel = curY - (layerHeight - heightrel)/ 2;

					if (update) {
						e.updateRelLocal(xrel, yrel, widthrel, heightrel);
					} else {
					  e.setXRelLocalBounded(xrel);
					  e.setYRelLocalBounded(yrel);
					}
					curX += boxWidth;
				} // if
			} // for

			curY -= layerHeight;
			curX  = 0.01;

		} // for

	} // doLayout


  // The doLayout method executes the Coffman-Graham Layer Assignment
  // algorithm and the Sugiyama algorithm on the boxes selected.
  // Assumption: All boxes selected are in the same container.

	public String doLayout(Diagram dg) 
	{
		EntityInstance	parent;

		m_ls.setLayouter(this);
		// get user's selection of boxes to be laid out

		Vector selectedBoxes = dg.getGroup();
		if (selectedBoxes == null) {
			  beep();
			  return "No group selected";
		}

		parent = parentOfSet(selectedBoxes);
		if (parent == null) {
			return	"Sugiyama requires that all things laid out share same parent";
		}
		dg.beginUndoRedo("Sugiyama layout");
		doLayout1(selectedBoxes, parent, true /* Updating */);
		dg.endUndoRedo();
		return "Graph redrawn with Coffman-Graham-Sugiyama method";

	} // doLayout

	public void processKeyEvent(int key, int modifiers, Object object) 
	{
		Diagram	dg;
		String	rmsg;

		dg = m_ls.getDiagram();
		if (dg != null) {
			rmsg = doLayout(dg);
			m_ls.doFeedback(rmsg);
			dg.rescaleDiagram();
			m_ls.repaintDg();
	}	}

} // SugiyamaLayout






